function [vertices, faces] = calcCylinder(r, R, Radius, Height, SideCount)
% calcBlock.m
%
% Return vertices and faces of a cylinder geometry
% Input
% r 		: position of the center of cylinder (3x1)
% R 		: rotation matrix (3x3)
% Radius	: radius of the cylinder
% Height	: height of the cylinder
% SideCount	: number of faces to simulate the lateral surface of the cylinder
% Output
% vertices	: position of the vertices ((2*SideCount)x3)
% faces 	: a table of the vertex numbers that make up each face of the cylinder
%      .side	a table for the lateral surface (SideCountx4)
%      .bttm	a table for the bottom and top surfaces (2x4)
% 
% Note. vertices are expressed in the body fixed frame.
%
% Ref. https://www.sky-engin.jp/MATLABAnimation/chap08/chap08.html
%
% Revision history
% 250507 Drafted
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Vertices
vertices_0 = zeros(2*SideCount, 3);
for i = 1:SideCount
    theta = 2*pi/SideCount*(i-1);
    vertices_0(i,:) = [Radius*cos(theta), Radius*sin(theta), 0];
    vertices_0(SideCount+i,:) = [Radius*cos(theta), Radius*sin(theta), Height];
end
vertices_0 = vertices_0 - ones(size(vertices_0,1),1)*[0,0,Height/2];
vertices = ones(size(vertices_0,1),1)*r' + vertices_0*R';

% Side faces
sideFaces = zeros(SideCount, 4);
for i = 1:(SideCount-1)
    sideFaces(i,:) = [i, i+1, SideCount+i+1, SideCount+i];
end
sideFaces(SideCount,:) = [SideCount, 1, SideCount+1, 2*SideCount];

% Bottom face and top face
bottomFaces = [
    1:SideCount;
    (SideCount+1):2*SideCount];
% faces
faces.side = sideFaces;
faces.bttm = bottomFaces;
