function [u,user_message] = Control(t_in,x_in)
% Control.m
%
% user-defined control algorithm
%
% INPUT
%	t_in	: current time [s]
%	x_in	: state variables (13x1 vector)
% OUTPUT
%	u		: thruster torque (x,y,z) [Nm] and wheel torque (y,z) [Nm] (5x1 vector)
%	user_message	: string of letters to be displayed in the real time monitor
%
% Revision history
% 250507 Drafted
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Please design your OWN control algorithm.
% Here is an example.
%
% Please DO NOT use toolbox functions.
% Please define YOUR functions in the "user" folder.
%
% user can use global variable under the name of user
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global N_q N_omg N_h_B N_fuel N_pointing
global user
    
%%--- load state variables
q		= x_in(N_q);
omg		= x_in(N_omg);
h_B		= x_in(N_h_B);

%%--- load environment property
s_I		= user.s_I;
g_I		= user.g_I;
angle_tol	= user.angle_tol;

%%--- load s/c property
I_B		= user.I_B;
n_B		= user.n_B;
r_B		= user.r_B;
hw_max_y	= user.hw_max_y;
hw_max_z	= user.hw_max_z;
trq_max	= user.trq_max;
thr_max	= user.thr_max;
thr_arm	= user.thr_arm;

%%--- set user initial state
if ( t_in==0 ),
	user.mode	= 'pointing';
	user.pointing	= 0;
end

%%--- set user variables
th_torque	= zeros(3,1);
wh_torque	= zeros(3,1);

C_I2B		= q2DCM(q);
g_B			= C_I2B*g_I;
angle_now	= acos(n_B'*g_B);
%% the range of acos is [0,pi]

%%--- mode check
if ( strcmp(user.mode,'pointing') ),
	if ( abs(angle_now) < angle_tol/2 ),
		user.pointing	= user.pointing + 1;
		if ( user.pointing>1000 );
			user.mode	= 'stationary';
		end
	end
end

if ( norm(cross(n_B,g_B)) > 0 ),
	trq_axis	= cross(n_B,g_B)/norm(cross(n_B,g_B));
else
	w_B			= randn(3,1);
	trq_axis	= cross(n_B,w_B)/norm(cross(n_B,w_B));
end
Kp			= (0.1)^2/2;
Kd			= 0.1;

%%--- orient the s/c using thrusters and then switch to wheel based control
if ( strcmp(user.mode,'pointing') ),
	if ( angle_now >= angle_tol/2 ),
		th_torque	= I_B*(Kp*angle_now*trq_axis+Kd*(-omg));
	else
		th_torque	= I_B*(Kd*(0-omg));
	end
	if ( max(abs(th_torque))>(thr_max*thr_arm) ),
		th_torque	= th_torque/norm(th_torque)*(thr_max*thr_arm);
	end
else
	if ( angle_now >= angle_tol/2 ),
		wh_torque	= I_B*(Kp*angle_now*trq_axis+Kd*(-omg));
	else
		wh_torque	= I_B*(Kd*(0-omg));
	end
	s_B		= C_I2B*s_I;
	F_B		= SRP_force(n_B,s_B);	%% expressed in body frame
	wh_torque	= wh_torque + cross(omg,I_B*omg+h_B) - cross(r_B,F_B);	%% feedforward 
	%% check momentum stored in each wheel
	%% if it approaches the limit, unload it by thrusters
	if ( h_B(2)> hw_max_y ),
		th_torque(2)	= -min(trq_max,thr_max*thr_arm);
		wh_torque(2)	= -th_torque(2);
	elseif ( h_B(2)<-hw_max_y ),
		th_torque(2)	=  min(trq_max,thr_max*thr_arm);
		wh_torque(2)	= -th_torque(2);
	end
	if ( h_B(3)> hw_max_z ),
		th_torque(3)	= -min(trq_max,thr_max*thr_arm);
		wh_torque(3)	= -th_torque(3);
	elseif ( h_B(3)<-hw_max_z ),
		th_torque(3)	=  min(trq_max,thr_max*thr_arm);
		wh_torque(3)	= -th_torque(3);
	end
	for j = 2:3,
		if ( wh_torque(j)> trq_max ),
			th_torque(j)	= wh_torque(j)-trq_max;
			wh_torque(j)	= trq_max;
		elseif ( wh_torque(j)<-trq_max ),
			th_torque(j)	= wh_torque(j)+trq_max;
			wh_torque(j)	=-trq_max;
		else
		end
	end
	%% check that there is no wheel along x-axis
	th_torque(1)	= wh_torque(1);
	wh_torque(1)	= 0;
end

%% set output variables
u		= [th_torque;wh_torque(2);wh_torque(3)];
user_message	= sprintf('%s',user.mode);

